#!/usr/bin/env python3
# encoding: utf-8
#
# Copyright (C) 2020 Timm Fitschen <t.fitschen@indiscale.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
#
"""
Confirm a loan.
"""
from __future__ import absolute_import

import linkahead as db
from caosadvancedtools.serverside.helper import get_timestamp, print_success

from .box_loan import (BOX, BOX_BORROWED, DESTINATION, F_LOAN, LENT, S_LENT, S_LOAN_ACCEPTED,
                       get_borrower_names, get_loan, main, set_location_of_borrowed_items,
                       set_property, set_references_to_current_version, update_loan_state)


def _set_lent_box(loan):
    """ Store the exact versions of borrowed items when it was delivered to the borrower. """
    references = set_references_to_current_version(loan.get_property(BOX).value)
    borrowed_prop = db.get_entity_by_name(BOX_BORROWED.name, role="PROPERTY")
    loan.add_property(id=borrowed_prop.id, value=references,
                      datatype=db.LIST(BOX.name) if isinstance(references, list) else BOX.name)
    loan.update()


def set_loan_location(loan):
    """Set the location of the box to the return location of the loan if configured."""

    config = db.get_config()
    if not ("Misc" in config and "entity_loan.no_location_updates" in config["Misc"]
            and config["Misc"]["entity_loan.no_location_updates"]):
        items = set_location_of_borrowed_items(loan, DESTINATION)
        items.update()


def _confirm_loan(data):
    """Update a loan Record and add the `lent` Property."""
    loan = get_loan(data[F_LOAN])
    # This changes the state from "loan_accepted" to "lent".
    update_loan_state(loan, S_LENT)

    _set_lent_box(loan)

    # updates the box location
    set_loan_location(loan)

    return loan


def confirm_loan(data):
    """Confirm a loan.

    I.e. update the `Loan` Record and add the `lent` Property.
    """
    loan = _confirm_loan(data)
    fn, ln = get_borrower_names(loan)
    loan = get_loan(data[F_LOAN])
    box_id = loan.get_property(BOX).value

    if not isinstance(box_id, list):
        box_id = [box_id]
    print_success('Thank you for confirming the loan request by {fn} {ln}.<br>'
                  'The Location of the Box was updated. '
                  '<a href="/Entity/{bid}" title="Reload this page.">Reload</a> '
                  'to view the new Location.<br>'
                  'You can also checkout the new loan state '
                  '<a href="{loan}" title="Go to the confirmed loan '
                  'record.">here</a>'.format(fn=fn, ln=ln, loan=loan.id,
                                             bid="&".join([str(el) for el in box_id])))

    return 0


if __name__ == "__main__":
    main(confirm_loan)
